#ifdef __USAGE

Uptime v1.1d - Show node uptime, system load, # of users.
Copyright (C) 1994 Trever Miller 

Usage:

%C [ -a | -n NODE ]  [ -i IDLETOP ]

             or

%C -l


where  NODE      Node to get uptime for (default this one)
       IDLETOP   Highest priority to consider part of "idle" (default 4)
       -a        Show all nodes 
       -l        Show License information

Invoking this program as "ruptime" is equivalient to using the "-a" option.

It is suggested that "sac -i 9" or higher be used so that the idle %%
better represents the general system load over the last minute.

If you have any suggestions or changes to this program, please send them
to the author at :

            quics    - tamiller
            internet - trever.miller@cyberdex.cuug.ab.ca

#endif

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <string.h>

#include <sys/vc.h>
#include <sys/kernel.h>
#include <sys/osinfo.h>
#include <sys/osstat.h>
#include <sys/psinfo.h>
#include <sys/sidinfo.h>

#ifdef __STDC__
# define    P(s) s
#else
# define P(s) ()
#endif

int print_uptime P((nid_t node ));

void print_idle P((nid_t node ));

void show_gpl P((void));

#undef P

#define IdleTopDefault 4

char  ProgName[80]="";
nid_t OurNode  = 0;
int   MaxNodes = 1;
int   IdleTop  = IdleTopDefault;

int main(int argc, char **argv)
{
    extern char   *optarg;

    extern int     optind, 
                   opterr, 
                   optopt;

    struct _osinfo osinfo;

    nid_t          node      = 0;

    int            all_nodes = 0,
                   c         = 0,
                   exit_code = EXIT_FAILURE;

    strncpy(ProgName,basename(argv[0]),sizeof(ProgName)-1);

    if(qnx_osinfo(0,&osinfo) == -1)
    {
       perror(ProgName);
       exit(EXIT_FAILURE);
    }

    OurNode = (nid_t) osinfo.nodename;
    MaxNodes = osinfo.max_nodes;

    if(strcmp(argv[0],"ruptime") == EXIT_SUCCESS)
        ++all_nodes;

	while((c = getopt(argc, argv,"haln:i:")) != -1)
	{
		switch(c)
		{
			case 'a': ++all_nodes                      ; break;

			case 'n': if((node = qnx_strtonid(optarg,NULL)) == -1)
			          {
			            perror(ProgName);
			            exit(EXIT_FAILURE);
			          } 
			          break;

			case 'i': IdleTop =         atoi(optarg) ; break;

			case 'l': show_gpl(); exit(EXIT_SUCCESS) ; break;

			case 'h':
			case '?': print_usage(argv); 
					  exit(EXIT_FAILURE)             ; break;

			default :                                  break;
		} /* switch() */
	} /* while() */

	printf("\nUptime version 1.1d, Copyright (C) 1994 Trever Miller\n\n");

    if(node == 0)
        node = OurNode;   /* default to our node number */

    if(IdleTop > 32)
    {
        IdleTop = IdleTopDefault;

        fprintf(stderr,"%s: Idletop too high. Using default of %d\n",
                ProgName, IdleTopDefault);
    }

    if(node < 0 || node > MaxNodes)
    {
        fprintf(stderr,"%s: Invalid node number \"%d\".",ProgName,node);
        fprintf(stderr," (You are licenced for %d nodes.)\007\n", MaxNodes);
        exit(EXIT_FAILURE);
    }

    if(all_nodes)
    {
        for(node = 1; node <= MaxNodes; node++)
            print_uptime(node);

        exit_code = EXIT_SUCCESS;
    }
    else
        exit_code = print_uptime(node);

    return(exit_code);

} /* main() */


int print_uptime(nid_t node)
{
    struct _sidinfo sidinfo;

    int    user_count = 0,
           sid        = 0;

    struct _psinfo psdata;

    pid_t  vid       = 0;

    int    days      = 0,
           hours     = 0;

    time_t now       = 0,
           boot_time = 0,
           uptime    = 0;

    char   time_str[80] = "";

    int    exit_code = EXIT_FAILURE;

    int status = -1;


    if(node == 0)
        node = OurNode;

    if(node == OurNode)
    {
        vid = PROC_PID;
    }
    else
    {
        vid = qnx_vc_attach(node, PROC_PID, 1000, 0);
    }

    printf("Node %3d    ",node);

    if((status = qnx_psinfo(vid, PROC_PID, &psdata, 0, NULL)) == -1)
    {
        printf("down\n");
    }
    else
    {
        boot_time = psdata.un.proc.start_time;

        time(&now);
        uptime = now - boot_time;

        hours   = uptime/3600L;
        uptime %= 3600L;

        if(hours>23)
        {
            days   = hours / 24;
            hours %= 24;
        }

        strftime(time_str,sizeof(time_str)-1,"Booted %d-%b-%y %H:%M %Z",
                 localtime(&boot_time));

        if(days > 0)
            printf("up %2d+",days);
        else
            printf("up    ");

        printf("%02d:%02d,   %s,   ", hours, uptime/60, time_str);

        print_idle(node);

        for(sid=0; (sid=qnx_sid_query(vid,sid,&sidinfo)) != -1; ++sid)
        {
            if( (strcmp("System",sidinfo.name) != 0) &&
                (strlen(sidinfo.tty_name) > 0) &&
                (strstr(sidinfo.tty_name,"/dev/null")) == NULL 
              )
                user_count++;
        }

        if(user_count>0)
            printf(",   %d user%s.",user_count,(user_count>1)?"s":"");

        printf("\n");

        exit_code = EXIT_SUCCESS;
    }

    qnx_vc_detach(vid);

    return(exit_code);

} /* print_uptime() */


void print_idle(nid_t node)
{
    struct _osstat osstat;

    unsigned short i, 
                   maxv, 
                   idle; 

    if(qnx_osstat(node,&osstat) == -1)
    {
        perror("qnx_osstat");
        exit(EXIT_FAILURE);
    }

    idle = 0;

    for(maxv = i = 0; i < 32 ; ++i)
        maxv += osstat.cpu_load[i];

    if(maxv == 0)
       maxv = 1;

    for(i = 0; i <= IdleTop ; ++i)
        idle += (osstat.cpu_load[i]*100) / maxv;

    printf("%2d%% idle",idle);

    return;

} /* print_idle() */

